@@ -127,13 +127,16 @@ class MenuHandler:
             pass
 
     def show(self):
-        self.rebuild()
-        self.panel.top()
-        #self.panel.move(0,0)
-        #self.panel.move(self.menuOriginLine, 0)
-        #self.panel.move(self.menuOriginLine, self.menu.x)
-        self.panel.move(self.menuOriginLine, self.menu.y)
-        self.panel.show()
+        try:
+            self.rebuild()
+            self.panel.top()
+            #self.panel.move(0,0)
+            #self.panel.move(self.menuOriginLine, 0)
+            #self.panel.move(self.menuOriginLine, self.menu.x)
+            self.panel.move(self.menuOriginLine, self.menu.y)
+            self.panel.show()
+        except: # The window was probably too short, so panel.move() returns ERR.
+            return False
         self.curses_win.keypad(True)
         curses.panel.update_panels()
         curses.doupdate()
@@ -183,7 +186,10 @@ class MenuHandler:
                     if self.menu.items[item]["has_submenu"]:    # If it opens a sub-menu..
                         # Keep it on the screen.
                         # Redraw previously selected as normal:
-                        curses_addstr(self.curses_win, current_option + 1, 2, options[current_option], menuItemColor)
+                        if not self.menu.title:
+                            curses_addstr(self.curses_win, current_option + 1, 2, options[current_option], menuItemColor)
+                        else:
+                            curses_addstr(self.curses_win, current_option + 2, 2, options[current_option], menuItemColor)
                         # Then highlight the new one.
                         current_option = options.index(item)
                         #self.rebuild()
@@ -214,6 +220,8 @@ class MenuHandler:
                 self.hide()
                 prompting = False
                 response = "Left"
+                if self.menu.is_submenu:
+                    response = "Pop"
                 #self.hide()
                 #prompting = False
                 # Here: Launch a different menu
@@ -323,6 +331,7 @@ class DrawCharPickerHandler:
         #pdb.set_trace()
         self.window.addstr(maxLines - 3, 0, "Enter a character to use for drawing: ")
         prompting = True
+        curses.flushinp()
         while prompting:
             c = self.window.getch()
             time.sleep(0.01)
@@ -356,41 +365,41 @@ class DrawCharPickerHandler:
             elif c in [curses.KEY_F10]:
                 self.caller.appState.drawChar = chr(self.caller.caller.caller.chMap['f10'])
                 prompting = False
-            elif c == 27:   # esc, cancel
+            elif c in [27, 13, curses.KEY_ENTER]:   # 27 = esc, 13 = enter, cancel
                 prompting = False
             else:
-                self.caller.appState.drawChar = chr(c)
-                prompting = False
+                try:
+                    if chr(c).isprintable():
+                        newChar = chr(c)
+                        self.caller.appState.drawChar = newChar
+                        prompting = False
+                except:
+                    pass
+                pass
         #self.caller.caller.drawCharPickerButton.label = self.caller.appState.drawChar
         self.caller.caller.drawCharPickerButton.set_label(self.caller.appState.drawChar)
         self.window.addstr(maxLines - 3, 0, "                                          ")
         self.caller.caller.caller.refresh()
 
 class ColorPickerHandler:
-    def __init__(self, colorPicker, window):
+    def __init__(self, colorPicker, window, width=38, height=8):
         self.colorPicker = colorPicker
+        self.colorMode = colorPicker.colorMode  # "256" or "16"
+        self.totalColors = colorPicker.totalColors
         self.parentWindow = window
         self.x = colorPicker.x
         self.y = colorPicker.y
         self.parentWindow = colorPicker.caller.stdscr
         self.appState = colorPicker.caller.appState
+        self.ansi = colorPicker.caller.ansi
         # figure out picker size
         #total = curses.COLORS
         #total = curses.COLORS
         realmaxY,realmaxX = self.parentWindow.getmaxyx()
         self.realmaxY = realmaxY
         self.realmaxX = realmaxX
-        #self.height = int(total / realmaxX) + 2 # enough lines to fill content+
-        #self.height = 4
-        self.height = 8
-        #height = int(total / realmaxY) # enough lines to fill content+
-        #self.width = realmaxX - 10 
-        #self.width = 78
-        #self.width = 38 
-        self.width = 38 
-        #gridLine = [[]] * self.width
-        #self.colorGrid = [gridLine] * self.height
-        #self.colorGrid = [[0] * self.width] * self.height
+        self.height = height
+        self.width = width
         self.colorGrid = [[0 for i in range(self.width)] for j in range(self.height)]
         self.window = curses.newwin(self.height, self.width, self.x, self.y)
         self.window.keypad(True)
@@ -465,18 +474,37 @@ class ColorPickerHandler:
     def updateFgPicker(self):
         line = 0
         col = 1
+        if self.colorMode == "16":
+            col = 0
+        if self.colorMode == "256":
+            plain_color_pair = curses.color_pair(9)
+        elif self.colorMode == "16":
+            plain_color_pair = curses.color_pair(8)
         #maxWidth = self.realmaxX
         #maxHeight = self.realmaxY
         #for fg in range(0,curses.COLORS):  # 0-255
         width_counter = 0   # for color block width
-        for fg in range(1,self.appState.totalFgColors+1):  # 0-255
-            color_pair = curses.color_pair(fg)
+        #for fg in range(1,self.appState.totalFgColors+1):  # 0-255
+        firstColor = 1
+        if self.colorPicker.appState.iceColors:
+            firstColor = 0
+        for fg in range(firstColor,self.totalColors+1):  # 0-255
+            #color_pair = curses.color_pair(fg)
+            if self.colorMode == "256":
+                color_pair = curses.color_pair(fg)
+            elif self.colorMode == "16":
+                try:
+                    color_pair_number = self.ansi.colorPairMap[(fg, 0)]
+                    color_pair = curses.color_pair(color_pair_number)
+                except KeyError:
+                    pdb.set_trace()
             if col >= self.width - 2:
                 col = 0
                 line += 1
-            if fg == 16:    # first color for fancy displayed block color palette thing
-                line += 1
-                col = 0
+            if self.colorMode == "256":
+                if fg == 16:    # first color for fancy displayed block color palette thing
+                    line += 1
+                    col = 0
             if fg > 16:
                 width_counter += 1
             if width_counter == 36:
@@ -490,20 +518,106 @@ class ColorPickerHandler:
             #    pdb.set_trace()
             #curses_addstr(self.window, self.colorPicker.y + line, self.colorPicker.x + col, chr(self.fillChar), color_pair)
             # if fg == app's current fg, draw it as a * instead of self.fillChar
-            if fg == self.colorPicker.caller.colorfg:
+            bg = self.colorPicker.caller.colorbg
+            if self.colorMode == "16":
+                bg += 1
+                if bg == 8:
+                    bg = 0
+            if fg == self.colorPicker.caller.colorfg or fg == 0 and colorbg == 8:
                 if fg == 1: # black
-                    plain_color_pair = curses.color_pair(9)
                     if self.appState.colorPickerSelected:
-                        curses_addstr(self.window, line, col, 'X', plain_color_pair | curses.A_UNDERLINE | curses.A_BLINK)
+                        if fg == bg:
+                            curses_addstr(self.window, line, col, 'X', plain_color_pair | curses.A_UNDERLINE | curses.A_BOLD)
+                        else:
+                            curses_addstr(self.window, line, col, 'F', plain_color_pair | curses.A_UNDERLINE | curses.A_BOLD)
                     else:
-                        curses_addstr(self.window, line, col, 'X', plain_color_pair)
+                        if fg == bg:
+                            curses_addstr(self.window, line, col, 'X', plain_color_pair)
+                        else:
+                            curses_addstr(self.window, line, col, 'F', plain_color_pair)
                 else:
                     if self.appState.colorPickerSelected:
-                        curses_addstr(self.window, line, col, 'X', color_pair | curses.A_UNDERLINE | curses.A_BLINK)
+                        if self.colorMode == "256":
+                            if fg == bg:
+                                curses_addstr(self.window, line, col, 'X', color_pair | curses.A_UNDERLINE | curses.A_BOLD)
+                            else:
+                                curses_addstr(self.window, line, col, 'F', color_pair | curses.A_UNDERLINE | curses.A_BOLD)
+                        if self.colorMode == "16":
+                            if fg > 8:
+                                curses_addstr(self.window, line, col, 'F', color_pair | curses.A_UNDERLINE | curses.A_BOLD)
+                            else:
+                                curses_addstr(self.window, line, col, 'F', color_pair | curses.A_UNDERLINE )
                     else:
-                        curses_addstr(self.window, line, col, 'X', color_pair)
+                        if self.colorMode == "256":
+                            if fg == bg:
+                                curses_addstr(self.window, line, col, 'X', color_pair | curses.A_BOLD)
+                            else:
+                                curses_addstr(self.window, line, col, 'F', color_pair | curses.A_BOLD)
+                        if self.colorMode == "16":
+                            if fg == bg:
+                                if fg > 8:
+                                    curses_addstr(self.window, line, col, 'X', color_pair | curses.A_BOLD)
+                                else:
+                                    curses_addstr(self.window, line, col, 'X', color_pair)
+                            else:
+                                if fg > 8:
+                                    curses_addstr(self.window, line, col, 'F', color_pair | curses.A_BOLD)
+                                else:
+                                    curses_addstr(self.window, line, col, 'F', color_pair)
+
+            # 16 color, black background (8), showing color 1 (black). . why the fuck is it 9 lol
+            elif self.colorMode == "16" and fg == 1 and bg == 9:
+                if self.appState.colorPickerSelected:
+                    curses_addstr(self.window, line, col, 'B', plain_color_pair | curses.A_UNDERLINE)
+                else:
+                    curses_addstr(self.window, line, col, 'B', plain_color_pair)
+            # 16 color, black background (8), showing color 8 (grey)
+            elif self.colorMode == "16" and fg == 9 and bg == 9:
+                # draw fill character unmodified
+                if fg > 8:
+                    curses_addstr(self.window, line, col, self.fillChar, color_pair | curses.A_BOLD)
+                else:
+                    curses_addstr(self.window, line, col, self.fillChar, color_pair)
+
+            elif fg == bg:
+                if self.appState.colorPickerSelected:
+                    if self.colorMode == "256":
+                        curses_addstr(self.window, line, col, 'B', color_pair | curses.A_UNDERLINE)
+                    elif self.colorMode == "16" and fg == 9:  # black, so show as default color
+                        curses_addstr(self.window, line, col, 'B', plain_color_pair | curses.A_UNDERLINE)
+                    elif self.colorMode == "16" and fg == 0:  # black, so show as default color
+                        curses_addstr(self.window, line, col, 'B', plain_color_pair | curses.A_UNDERLINE)
+                    elif self.colorMode == "16" and bg != 0:
+                        curses_addstr(self.window, line, col, 'B', color_pair | curses.A_UNDERLINE)
+                else:
+                    if self.colorMode == "256":
+                        curses_addstr(self.window, line, col, 'B', color_pair | curses.A_BOLD)
+                    elif self.colorMode == "16" and fg == 1:  # black, so show as default color
+                        curses_addstr(self.window, line, col, 'B', plain_color_pair)
+                    elif self.colorMode == "16" and fg == 9:  # black, so show as default color
+                        curses_addstr(self.window, line, col, 'B', plain_color_pair)
+                    elif self.colorMode == "16" and bg != 0:
+                        curses_addstr(self.window, line, col, 'B', color_pair | curses.A_UNDERLINE)
+                    #if self.colorMode == "16" and fg == 8:  # black, so show as default color
+                    #    curses_addstr(self.window, line, col, 'B', plain_color_pair)
+                    #elif self.colorMode == "16":
+                    #    curses_addstr(self.window, line, col, 'B', color_pair)
             else:
-                curses_addstr(self.window, line, col, self.fillChar, color_pair)
+                if self.colorMode == "256":
+                    curses_addstr(self.window, line, col, self.fillChar, color_pair)
+                elif self.colorMode == "16":
+                    # draw bright colors in 16 color mode
+                    if fg > 8:
+                        curses_addstr(self.window, line, col, self.fillChar, color_pair | curses.A_BOLD)
+                        #debug_string = str(color_pair)
+                        #self.colorPicker.caller.notify(debug_string)
+                    else:
+                        curses_addstr(self.window, line, col, self.fillChar, color_pair)
+            if self.colorMode == "16" and bg == 0 and fg == 0:    # black bg in 16 color mode
+                if self.appState.colorPickerSelected:
+                        curses_addstr(self.window, line, col, 'B', plain_color_pair | curses.A_UNDERLINE)
+                else:
+                        curses_addstr(self.window, line, col, 'B', plain_color_pair)
             col += 1
         curses_addstr(self.window, line, col + 5, "     ", color_pair)
         curses_addstr(self.window, line, col + 5, str(self.colorPicker.caller.colorfg), color_pair)
@@ -512,6 +626,30 @@ class ColorPickerHandler:
         #self.colorPicker.caller.notify(f"showFgPicker")
         self.showColorPicker(type="fg")
 
+    def move_up_256(self):
+        if self.colorMode == "256":
+            color = self.colorPicker.caller.colorfg
+            if color < 32:
+                color -= 16
+            elif color > 31 and color < 52:
+                color = 15
+            else:
+                color -= self.width - 2
+            if color < 1:
+                color = 1
+            self.colorPicker.caller.setFgColor(color)
+
+    def move_down_256(self):
+        if self.colorMode == "256":
+            color = self.colorPicker.caller.colorfg
+            if color < 16:
+                color += 16
+            else:
+                color += self.width - 2
+            if color >= self.totalColors:
+                color = self.totalColors
+            self.colorPicker.caller.setFgColor(color)
+
     def showColorPicker(self, type="fg"):
         #self.colorPicker.caller.notify(f"showColorPicker")
         """ Shows picker, has UI loop for letting user pick color with keyboard or mouse """
@@ -531,6 +669,7 @@ class ColorPickerHandler:
         color = self.colorPicker.caller.colorfg
         if self.appState.colorPickerSelected:
             prompting = True
+            #self.window.nodelay(0) # wait for input when calling getch
         else:
             prompting = False
         if self.appState.colorPickerSelected:
@@ -540,6 +679,10 @@ class ColorPickerHandler:
 
         #self.colorPicker.caller.notify(f"showColorPicker() hit. {prompting=}")
         
+        mov = self.colorPicker.caller.mov
+        appState = self.colorPicker.caller.appState
+        # ^ Used to determine if we clicked in the canvas:
+
         while(prompting):
             time.sleep(0.01)
             #self.colorPicker.caller.drawStatusBar()
@@ -547,7 +690,7 @@ class ColorPickerHandler:
             c = self.window.getch()
             if c in [98, curses.KEY_LEFT]:
                 if color == 0:
-                    color = curses.COLORS - 1
+                    color = self.totalColors
                 else:
                     color -= 1
                 #self.colorPicker.caller.colorfg = color
@@ -556,32 +699,41 @@ class ColorPickerHandler:
                 self.colorPicker.caller.drawStatusBar()
             elif c in [102, curses.KEY_RIGHT]:
                 color += 1
-                if color >= curses.COLORS:
+                #if color >= curses.COLORS:
+                if color > self.totalColors:
                     color = 0
                 #self.colorPicker.caller.colorfg = color
                 self.colorPicker.caller.setFgColor(color)
                 self.updateFgPicker()
                 self.colorPicker.caller.drawStatusBar()
             elif c == curses.KEY_UP:
-                if color < 32:
-                    color -= 16
-                elif color > 31 and color < 52:
-                    color = 15
-                else:
-                    color -= self.width - 2
+                if self.colorMode == "256":
+                    if color < 32:
+                        color -= 16
+                    elif color > 31 and color < 52:
+                        color = 15
+                    else:
+                        color -= self.width - 2
+                elif self.colorMode == "16":
+                    self.colorPicker.caller.nextBgColor()
+                    #color -= self.width - 2
                 if color <= 0:
                     color = 1
                 self.colorPicker.caller.setFgColor(color)
                 self.updateFgPicker()
                 self.colorPicker.caller.drawStatusBar()
             elif c == curses.KEY_DOWN:
-                if color < 16:
-                    color += 16
-                else:
-                    color += self.width - 2
-                if color >= curses.COLORS:
-                    color = curses.COLORS - 1
-                self.colorPicker.caller.setFgColor(color)
+                if self.colorMode == "256":
+                    if color < 16:
+                        color += 16
+                    else:
+                        color += self.width - 2
+                    if color >= self.totalColors:
+                        color = self.totalColors
+                    self.colorPicker.caller.setFgColor(color)
+                elif self.colorMode == "16":
+                    self.colorPicker.caller.prevBgColor()
+                    #color += self.width - 2
                 self.updateFgPicker()
                 self.colorPicker.caller.drawStatusBar()
             elif c == curses.KEY_HOME:
@@ -590,18 +742,22 @@ class ColorPickerHandler:
                 self.updateFgPicker()
                 self.colorPicker.caller.drawStatusBar()
             elif c == curses.KEY_END:
-                color = 255
+                #color = 255
+                color = self.totalColors
                 self.colorPicker.caller.setFgColor(color)
                 self.updateFgPicker()
                 self.colorPicker.caller.drawStatusBar()
-            elif c in [13, curses.KEY_ENTER, 9, 353]:   # Return, Accept color. 9=tab, 353=shift-tab
+            elif c in [13, curses.KEY_ENTER, 9, 353, 27]:   # Return, Accept color. 9=tab, 353=shift-tab, 27 = esc
                 #if not self.appState.stickyColorPicker:
                 if not self.appState.sideBarShowing:
                     self.hide()
                 prompting = False
+                if c == 27: # esc, cancel
+                    self.colorPicker.caller.colorfg = oldColor
                 self.appState.colorPickerSelected = False
                 c = None
                 self.updateFgPicker()
+                self.hideBorder()
                 #self.colorPicker.caller.notify(f"{c=}, {prompting=}")
                 if not self.colorPicker.caller.playing:    # caller.caller is the main UI thing
                     self.window.nodelay(0)  # wait
@@ -633,6 +789,7 @@ class ColorPickerHandler:
                     self.updateFgPicker()
                     self.colorPicker.caller.drawStatusBar()
                 elif mouseY >= self.origin and mouseX > self.x and mouseX < self.x + len(self.colorGrid[0])-2:   # cpicked in the color picker
+                    self.hideBorder()
                     #self.colorPicker.caller.notify(f"DEBUG: self.origin={self.origin}, self.x = {self.x}. mouseX={mouseX}, mouseY={mouseY}", pause=True)
                     self.gotClick(mouseX, mouseY)
                     prompting = False
@@ -645,7 +802,11 @@ class ColorPickerHandler:
                     #        color = self.colorGrid[clickedLine][clickedCol]
                     #        self.colorPicker.caller.setFgColor(color)
                     #        self.updateFgPicker()
-
+                elif mouseY < mov.sizeY and mouseX < mov.sizeX \
+                        and mouseY + appState.topLine < appState.topLine + self.colorPicker.caller.statusBarLineNum:
+                    # we did click in the canvas.
+                    self.hideBorder()
+                    prompting = False
 
 
                 if not prompting:
@@ -656,6 +817,7 @@ class ColorPickerHandler:
             elif c == 27:  # normal esc, Cancel
                 c = self.window.getch()
                 if c == curses.ERR: # Just esc was hit, no other escape sequence
+                    self.hideBorder()
                     self.colorPicker.caller.setFgColor(oldColor)
                     self.updateFgPicker()
                     if not self.appState.sideBarShowing:
@@ -676,12 +838,15 @@ class ColorPickerHandler:
         #self.hide()
         curses_cursorOn()
         self.window.nodelay(0)
+        if self.colorPicker.caller.playing:
+            self.window.nodelay(1)
         #curses.mousemask(curses.REPORT_MOUSE_POSITION | curses.ALL_MOUSE_EVENTS)
         #print('\033[?1003h') # enable mouse tracking
         return color
 
     def gotClick(self, mouseX, mouseY):
-        if mouseY >= self.origin and mouseX < + self.x + len(self.colorGrid[0])-2:   # cpicked in the color picker
+        # If clicked on a color, set an FG color
+        if not self.colorPicker.hidden and mouseY >= self.origin and mouseX < + self.x + len(self.colorGrid[0])-2:   # cpicked in the color picker
             clickedCol = mouseX - self.x
             clickedLine = mouseY - self.origin
             if mouseY < self.origin + self.height:
@@ -690,6 +855,20 @@ class ColorPickerHandler:
                     self.colorPicker.caller.setFgColor(color)
                     self.updateFgPicker()
 
+    def gotDoubleClick(self, mouseX, mouseY):
+        # set a BG color
+        if not self.colorPicker.hidden and mouseY >= self.origin and mouseX < + self.x + len(self.colorGrid[0])-2:   # cpicked in the color picker
+            clickedCol = mouseX - self.x - 1
+            clickedLine = mouseY - self.origin
+            if mouseY < self.origin + self.height:
+                if self.colorGrid[clickedLine][clickedCol] != 0:
+                    color = self.colorGrid[clickedLine][clickedCol]
+                    if self.colorPicker.caller.appState.colorMode == "16" and color < 9:
+                        # Only set BG color if it's one of the non-bright colors. (Sorry, no ice yet)
+                        self.colorPicker.caller.setBgColor(color)
+                        self.updateFgPicker()
+
+
 
     def update(self):
         curses.panel.update_panels()
